@@ -13,8 +13,8 @@ This module is based heavily on code from the Quixote ptl_import module by MEMS
 Exchange -- thanks guys.
 """
 
-__revision__ = "$Rev: 89 $"
-__date__ = "$Date: 2005-03-04 16:26:52 -0500 (Fri, 04 Mar 2005) $"
+__revision__ = "$Rev: 214 $"
+__date__ = "$Date: 2005-11-21 22:51:13 -0500 (Mon, 21 Nov 2005) $"
 __author__ = "Ryan Tomayko (rtomayko@gmail.com)"
 __copyright__ = "Copyright 2004-2005, Ryan Tomayko"
 
@@ -50,10 +50,10 @@ KID_EXT = kid.compiler.KID_EXT
 assert sys.hexversion >= 0x20000b1, "need Python 2.0b1 or later"
 
 _installed = False
-def install():
+def install(suffixes=None):
     global _installed
     if not _installed:
-        hooks = KIDHooks()
+        hooks = KIDHooks(suffixes=suffixes)
         loader = KIDLoader(hooks)
         if cimport is not None:
             importer = cModuleImporter(loader)
@@ -62,6 +62,11 @@ def install():
         ihooks.install(importer)
         _installed = True
 
+def uninstall():
+    global _installed
+    _installed = False
+    ihooks.uninstall()
+
 def import_template(name, filename, force=0):
     if not force and name and sys.modules.has_key(name):
         return sys.modules[name]
@@ -70,9 +75,14 @@ def import_template(name, filename, force=0):
     module = _create_module(code, name, filename)
     return module
 
+def get_template_name(name, filename):
+    if name:
+        return name
+    else:
+        return 'kid.util.template_' + hex(hash(filename) + 0x80000000)
+    
 def _create_module(code, name, filename, store=1):
-    if not name:
-        name = 'kid.util.' + hex(hash(filename))
+    name = get_template_name(name, filename)
     mod = sys.modules.get(name, None)
     if mod is None:
         mod = new.module(name)
@@ -87,11 +97,17 @@ KID_FILE = object()
 
 class KIDHooks(ihooks.Hooks):
 
+    def __init__(self, verbose=ihooks.VERBOSE, suffixes=None):
+        ihooks.Hooks.__init__(self, verbose)
+        self.suffixes = suffixes or []
+
     def get_suffixes(self):
-        return [(KID_EXT, 'r', KID_FILE)] + imp.get_suffixes()
+        return [(KID_EXT, 'r', KID_FILE)] \
+                + [(suffix, 'r', KID_FILE) for suffix in self.suffixes] \
+                + imp.get_suffixes()
 
 class KIDLoader(ihooks.ModuleLoader):
-    
+
     def load_module(self, name, stuff):
         file, filename, info = stuff
         (suff, mode, type) = info
