@@ -10,7 +10,7 @@ class Namespace(object):
             
     def qname(self, name):
         if self.prefix:
-            return self.prefix + ':name'
+            return self.prefix + ':' + name
         else:
             return name
         
