@@ -8,12 +8,17 @@ __author__ = "Ryan Tomayko (rtomayko@gmail.com)"
 __copyright__ = "Copyright 2004-2005, Ryan Tomayko"
 __license__ = "MIT <http://www.opensource.org/licenses/mit-license.php>"
 
+import re
+
 from kid.et import *
 from kid.pull import *
+from kid.pull import _coalesce
 
 # bring in well known namespaces
 import kid.namespace as namespace
 
+__all__ = ['doctypes', 'Serializer', 'XMLSerializer', 'HTMLSerializer']
+
 # some doctypes. you can pass strings from here or doctype tuples to
 # Serializers
 doctypes = {
@@ -31,11 +36,17 @@ class Serializer(object):
 
     namespaces = namespace.namespaces
     encoding = 'utf-8'
-
-    def __init__(self, encoding=None):
+    balanced_blocks = 1
+    strip_whitespace = 0
+    
+    def __init__(self, encoding=None, src_encoding="utf-8"):
         if encoding is not None:
             self.encoding = encoding
-    
+        self.src_encoding = src_encoding
+        
+    def has_only_pcdata(self, tagname):
+        return False
+
     def serialize(self, stream, encoding=None, fragment=0):
         text = list(self.generate(stream, encoding, fragment))
         return ''.join(text)
@@ -50,13 +61,54 @@ class Serializer(object):
     def generate(self, stream, encoding=None, fragment=0):
         pass
 
+    def apply_filters(self, stream):
+        stream = _coalesce(stream, self.src_encoding)
+        if self.strip_whitespace:
+            stream = self.whitespace_filter(stream)
+        else:
+            if self.balanced_blocks:
+                stream = self.balancing_filter(stream)
+        return stream
 
+    def balancing_filter(self, stream):
+        line_collapse = re.compile('\n{2,}')
+        text = ''
+        hops = 0
+        for ev, item in stream:
+            if ev == TEXT:
+                text = item
+                hops = 0
+            elif ev in (START, END) and item.tag != Fragment:
+                if hops > 0:
+                    if text and text.strip() == '':
+                        yield (TEXT, line_collapse.sub('\n', text))
+                elif text:
+                    if text.strip() == '':
+                        yield (TEXT, line_collapse.sub('\n', text))
+                    else:
+                        yield (TEXT, text)
+                yield (ev, item)
+                hops+=1
+                # pre-tag indentation was used.
+                # Make sure it's not used again for #PCDATA-sensitive elements
+                if ev == START and self.has_only_pcdata(item.tag):
+                    text = ''
+            else:
+                yield (ev, item)
+
+    def whitespace_filter(self, stream):
+        for ev, item in stream:
+            if ev == TEXT:
+                yield (TEXT, item.strip())
+            else:
+                yield (ev, item)
+        
 class XMLSerializer(Serializer):
 
     decl = 1
     doctype = None
     cdata_elements = []
-
+    
     def __init__(self, encoding=None, decl=None, doctype=None,
                  namespaces=None):
         Serializer.__init__(self, encoding)
@@ -69,7 +121,10 @@ class XMLSerializer(Serializer):
             self.doctype = doctypes[self.doctype]
         if namespaces:
             self.namespaces = namespaces
-        
+    
+    def can_be_empty_element(self, ns_stack, item_name):
+        return True
+    
     def generate(self, stream, encoding=None, fragment=0):
         """Serializes an event stream to bytes of the specified encoding.
         
@@ -83,7 +138,6 @@ class XMLSerializer(Serializer):
         escape_attrib = XMLSerializer.escape_attrib
         
         lastev = None
-        short_form = 0
         stream = iter(stream)
         names = NamespaceStack(self.namespaces)
         if not fragment:
@@ -91,12 +145,28 @@ class XMLSerializer(Serializer):
                 yield '<?xml version="1.0" encoding="%s"?>\n' % encoding
             if self.doctype is not None:
                 yield serialize_doctype(self.doctype) + '\n'
-        for ev, item in stream:
-            if lastev == START and ev != END:
+        text = None
+        for ev, item in self.apply_filters(stream):
+            if ev in (START, END) and item.tag == Fragment:
+                continue
+            elif ev == TEXT:
+                if text is not None:
+                    text = u''.join([text, item])
+                else:
+                    text = item
+                continue
+            if lastev == START:
+                if ev == END and (not text or not text.strip()) and self.can_be_empty_element(names, item.tag):
+                    yield ' />'
+                    lastev = END
+                    text = None
+                    names.pop()
+                    continue
                 yield ">"
-            if ev == TEXT and item:
-                yield escape_cdata(item, encoding)
-            elif ev == START:
+            if text:
+                yield escape_cdata(text, encoding)
+                text = None
+            if ev == START:
                 if item.tag == Comment:
                     yield "<!--%s-->" % item.text.encode(encoding)
                     lastev = COMMENT
@@ -105,8 +175,6 @@ class XMLSerializer(Serializer):
                     yield "<?%s?>" % item.text.encode(encoding)
                     lastev = PI
                     continue
-                elif item.tag == Fragment:
-                    continue
                 else:
                     tag = item.tag
                     names.push(namespaces(item, remove=1))
@@ -124,12 +192,9 @@ class XMLSerializer(Serializer):
                         else:
                             yield ' xmlns:%s="%s"' % (prefix.encode(encoding),
                                                       escape_attrib(uri, encoding))
-            elif ev == END and item.tag not in (Comment, ProcessingInstruction, Fragment):
-                if lastev == START:
-                    yield " />"
-                else:
-                    qname = names.qname(item.tag, default=1)
-                    yield "</%s>" % qname.encode(encoding)
+            elif ev == END and item.tag not in (Comment, ProcessingInstruction):
+                qname = names.qname(item.tag, default=1)
+                yield "</%s>" % qname.encode(encoding)
                 names.pop()
             lastev = ev
         return
@@ -189,6 +254,8 @@ class HTMLSerializer(Serializer):
     empty_elements = set(['area', 'base', 'basefont', 'br', 'col', 'frame',
                           'hr', 'img', 'input', 'isindex', 'link', 'meta',
                           'param'])
+    # These element tags should not be indented
+    elements_with_pcdata = set(['option', 'textarea', 'fieldset', 'title'])
     noescape_elements = set(['script', 'style'])
     boolean_attributes = set(['selected', 'checked', 'compact', 'declare',
                           'defer', 'disabled', 'ismap', 'multiple', 'nohref',
@@ -203,7 +270,12 @@ class HTMLSerializer(Serializer):
             self.doctype = doctypes[self.doctype]
         if transpose:
             self.transpose = transpose
-        
+
+    def has_only_pcdata(self, tagname):
+	if isinstance(tagname, (str,unicode)) and tagname[0] == '{':
+            tagname = tagname.split('}')[1] 
+	    return tagname in self.elements_with_pcdata
+
     def generate(self, stream, encoding=None, fragment=0):
         """Serializes an event stream to bytes of the specified encoding.
         
@@ -244,7 +316,7 @@ class HTMLSerializer(Serializer):
         stream = iter(stream)
         if not fragment and self.doctype is not None:
             yield serialize_doctype(self.doctype) + '\n'
-        for ev, item in stream:
+        for ev, item in self.apply_filters(stream):
             if ev == TEXT and item:
                 escape = current not in noescape_elements
                 yield escape_cdata(item, encoding, escape)
@@ -336,6 +408,25 @@ class HTMLSerializer(Serializer):
             _raise_serialization_error(text)
     escape_attrib = staticmethod(escape_attrib)
 
+class XHTMLSerializer(XMLSerializer):
+    empty_elements = [namespace.xhtml.clarkname(name) for name in HTMLSerializer.empty_elements]
+    elements_with_pcdata = [namespace.xhtml.clarkname(name) for name in HTMLSerializer.elements_with_pcdata]
+    
+    def can_be_empty_element(self, ns_stack, tagname):
+        return tagname in self.empty_elements
+
+    def has_only_pcdata(self, tagname):
+        return tagname in self.elements_with_pcdata
+
+class PlainSerializer(Serializer):
+
+    def generate(self, stream, encoding=None, fragment=0):
+        # XXX: Should this be ASCII?
+        encoding = encoding or self.encoding or 'utf-8'
+        for ev, item in self.apply_filters(stream):
+            if ev == TEXT:
+                yield item
+
 
 class NamespaceStack:
     
